@@ -1,23 +1,24 @@
-from astropy.table import Table
+from abc import ABCMeta, abstractmethod
 
 from pyobs.images import Image
 from pyobs.images.processor import ImageProcessor
 
 
-class SourceDetection(ImageProcessor):
+class SourceDetection(ImageProcessor, metaclass=ABCMeta):
     """Base class for source detection."""
     __module__ = 'pyobs.images.processors.detection'
 
-    def __call__(self, image: Image) -> Table:
+    @abstractmethod
+    async def __call__(self, image: Image) -> Image:
         """Find stars in given image and append catalog.
 
         Args:
             image: Image to find stars in.
 
         Returns:
-            Full table with results.
+            Image with attached catalog.
         """
-        raise NotImplementedError
+        ...
 
 
 __all__ = ['SourceDetection']
