@@ -1,21 +1,24 @@
+from abc import ABCMeta, abstractmethod
 from typing import List, Any
 
 from .IMotion import IMotion
 
 
-class IFilters(IMotion):
+class IFilters(IMotion, metaclass=ABCMeta):
     """The module can change filters in a device."""
     __module__ = 'pyobs.interfaces'
 
-    def list_filters(self, **kwargs: Any) -> List[str]:
+    @abstractmethod
+    async def list_filters(self, **kwargs: Any) -> List[str]:
         """List available filters.
 
         Returns:
             List of available filters.
         """
-        raise NotImplementedError
+        ...
 
-    def set_filter(self, filter_name: str, **kwargs: Any) -> None:
+    @abstractmethod
+    async def set_filter(self, filter_name: str, **kwargs: Any) -> None:
         """Set the current filter.
 
         Args:
@@ -24,15 +27,16 @@ class IFilters(IMotion):
         Raises:
             ValueError: If binning could not be set.
         """
-        raise NotImplementedError
+        ...
 
-    def get_filter(self, **kwargs: Any) -> str:
+    @abstractmethod
+    async def get_filter(self, **kwargs: Any) -> str:
         """Get currently set filter.
 
         Returns:
             Name of currently set filter.
         """
-        raise NotImplementedError
+        ...
 
 
 __all__ = ['IFilters']
