@@ -1,13 +1,15 @@
+from abc import ABCMeta, abstractmethod
 from typing import Tuple, Any
 
 from .interface import Interface
 
 
-class IPointingRaDec(Interface):
+class IPointingRaDec(Interface, metaclass=ABCMeta):
     """The module can move to RA/Dec coordinates, usually combined with :class:`~pyobs.interfaces.ITelescope`."""
     __module__ = 'pyobs.interfaces'
 
-    def move_radec(self, ra: float, dec: float, **kwargs: Any) -> None:
+    @abstractmethod
+    async def move_radec(self, ra: float, dec: float, **kwargs: Any) -> None:
         """Starts tracking on given coordinates.
 
         Args:
@@ -17,15 +19,16 @@ class IPointingRaDec(Interface):
         Raises:
             ValueError: If device could not track.
         """
-        raise NotImplementedError
+        ...
 
-    def get_radec(self, **kwargs: Any) -> Tuple[float, float]:
+    @abstractmethod
+    async def get_radec(self, **kwargs: Any) -> Tuple[float, float]:
         """Returns current RA and Dec.
 
         Returns:
             Tuple of current RA and Dec in degrees.
         """
-        raise NotImplementedError
+        ...
 
 
 __all__ = ['IPointingRaDec']
