@@ -1,13 +1,15 @@
+import asyncio
 import logging
 import time
 from typing import Optional, Union, Any
 
-from pyobs.interfaces.proxies import IWeatherProxy
+from pyobs.interfaces import IWeather
 from pyobs.modules import Module
 from pyobs.events import BadWeatherEvent, GoodWeatherEvent, Event
 from pyobs.interfaces import IMotion
 from pyobs.mixins import MotionStatusMixin
 from pyobs.utils.enums import MotionStatus
+from pyobs.utils.parallel import event_wait
 
 log = logging.getLogger(__name__)
 
@@ -16,24 +18,24 @@ class WeatherAwareMixin:
     """Mixin for IMotion devices that should park(), when weather gets bad."""
     __module__ = 'pyobs.mixins'
 
-    def __init__(self, weather: Optional[Union[str, IWeatherProxy]] = None, **kwargs: Any):
+    def __init__(self, weather: Optional[Union[str, IWeather]] = None, **kwargs: Any):
         self.__weather = weather
         self.__is_weather_good: Optional[bool] = None
         this = self
         if isinstance(self, Module):
-            self.add_thread_func(this.__weather_check, True)
+            self.add_background_task(this.__weather_check, True)
         else:
             raise ValueError('This is not a module.')
 
-    def open(self) -> None:
+    async def open(self) -> None:
         """Open mixin."""
         # subscribe to events
         this = self
         if isinstance(self, Module) and self.comm is not None:
-            self.comm.register_event(BadWeatherEvent, this.__on_bad_weather)
-            self.comm.register_event(GoodWeatherEvent, this._on_good_weather)
+            await self.comm.register_event(BadWeatherEvent, this.__on_bad_weather)
+            await self.comm.register_event(GoodWeatherEvent, this._on_good_weather)
 
-    def __on_bad_weather(self, event: Event, sender: str) -> bool:
+    async def __on_bad_weather(self, event: Event, sender: str) -> bool:
         """Abort exposure if a bad weather event occurs.
 
         Args:
@@ -50,15 +52,15 @@ class WeatherAwareMixin:
 
         # do we need to park?
         if isinstance(self, MotionStatusMixin) and isinstance(self, IMotion):
-            if self.get_motion_status() != MotionStatus.PARKED:
+            if await self.get_motion_status() != MotionStatus.PARKED:
                 log.warning('Received bad weather event, shutting down.')
-                self.park()
+                await self.park()
             return True
         else:
             log.error('This is not a MotionStatusMixin/IMotion.')
             return False
 
-    def _on_good_weather(self, event: Event, sender: str) -> bool:
+    async def _on_good_weather(self, event: Event, sender: str) -> bool:
         """Change status of weather.
 
         Args:
@@ -74,7 +76,7 @@ class WeatherAwareMixin:
         self.__is_weather_good = True
         return True
 
-    def __weather_check(self) -> None:
+    async def __weather_check(self) -> None:
         """Thread for continuously checking for good weather"""
 
         # module?
@@ -83,13 +85,13 @@ class WeatherAwareMixin:
             module = self
 
             # wait a little
-            self.closing.wait(10)
+            await asyncio.sleep(10)
 
             # time of last park attempt
             last_park_attempt = None
 
             # run until closing
-            while not module.closing.is_set():
+            while True:
                 # got a weather module?
                 if this.__weather is None:
                     # weather is always good
@@ -98,10 +100,10 @@ class WeatherAwareMixin:
                 else:
                     try:
                         # get proxy
-                        weather: IWeatherProxy = module.proxy(this.__weather, IWeatherProxy)
+                        weather: IWeather = await module.proxy(this.__weather, IWeather)
 
                         # get good status
-                        this.__is_weather_good = weather.is_weather_good().wait()
+                        this.__is_weather_good = await weather.is_weather_good()
 
                     except:
                         # could either not connect or weather is not good
@@ -110,9 +112,9 @@ class WeatherAwareMixin:
                 # if not good, park now
                 if isinstance(self, MotionStatusMixin) and isinstance(self, IMotion):
                     if this.__is_weather_good is False and \
-                            self.get_motion_status() not in [MotionStatus.PARKED, MotionStatus.PARKING]:
+                            await self.get_motion_status() not in [MotionStatus.PARKED, MotionStatus.PARKING]:
                         try:
-                            self.park()
+                            asyncio.create_task(self.park())
                             log.info('Weather seems to be bad, shutting down.')
                         except:
                             # only log, if last attempt is more than 60s ago
@@ -128,7 +130,7 @@ class WeatherAwareMixin:
                     raise ValueError('This is not a MotionStatusMixin/IMotion.')
 
                 # sleep a little
-                module.closing.wait(10)
+                await asyncio.sleep(10)
 
         else:
             # not a module
