@@ -1,4 +1,9 @@
+"""
+Modules for image operations.
+TODO: write doc
+"""
+__title__ = 'Image operations'
+
 from .imagewatcher import ImageWatcher
 from .imagewriter import ImageWriter
-from .onlinereduction import OnlineReduction
 from .seeing import Seeing
