@@ -7,7 +7,7 @@ import pandas as pd
 from astropy.coordinates import SkyCoord
 import astropy.units as u
 
-from pyobs.interfaces.proxies import IAcquisitionProxy, ITelescopeProxy
+from pyobs.interfaces import IAcquisition, ITelescope
 from pyobs.modules import Module
 from pyobs.comm import InvocationException
 from pyobs.interfaces import IAutonomous
@@ -58,21 +58,21 @@ class PointingSeries(Module, IAutonomous):
             self._az_range = (0., 360. - 360. / self._num_az)
 
         # add thread func
-        self.add_thread_func(self._run_thread, False)
+        self.add_background_task(self._run_thread, False)
 
-    def start(self, **kwargs: Any):
+    async def start(self, **kwargs: Any):
         """Starts a service."""
         pass
 
-    def stop(self, **kwargs: Any):
+    async def stop(self, **kwargs: Any):
         """Stops a service."""
         pass
 
-    def is_running(self, **kwargs: Any) -> bool:
+    async def is_running(self, **kwargs: Any) -> bool:
         """Whether a service is running."""
         return True
 
-    def _run_thread(self):
+    async def _run_thread(self):
         """Run a pointing series."""
 
         # create grid
@@ -87,11 +87,11 @@ class PointingSeries(Module, IAutonomous):
         grid = pd.DataFrame(grid).set_index(['alt', 'az'])
 
         # get acquisition and telescope units
-        acquisition: IAcquisitionProxy = self.proxy(self._acquisition, IAcquisitionProxy)
-        telescope: ITelescopeProxy = self.proxy(self._telescope, ITelescopeProxy)
+        acquisition = await self.proxy(self._acquisition, IAcquisition)
+        telescope = await self.proxy(self._telescope, ITelescope)
 
         # loop until finished
-        while not self.closing.is_set():
+        while True:
             # get all entries without offset measurements
             todo = list(grid[~grid['done']].index)
             if len(todo) / len(grid) < self._finish:
@@ -104,10 +104,6 @@ class PointingSeries(Module, IAutonomous):
 
             # try to find a good point
             while True:
-                # aborted or not running?
-                if self.closing.is_set():
-                    return
-
                 # pick a random index and remove from list
                 alt, az = random.sample(todo, 1)[0]
                 todo.remove((alt, az))
@@ -137,10 +133,10 @@ class PointingSeries(Module, IAutonomous):
             # acquire target and process result
             try:
                 # move telescope
-                telescope.move_radec(float(radec.ra.degree), float(radec.dec.degree)).wait()
+                await telescope.move_radec(float(radec.ra.degree), float(radec.dec.degree))
 
                 # acquire target
-                acq = acquisition.acquire_target().wait()
+                acq = await acquisition.acquire_target()
 
                 #  process result
                 if acq is not None:
@@ -154,10 +150,7 @@ class PointingSeries(Module, IAutonomous):
             grid.loc[alt, az] = True
 
         # finished
-        if self.closing.is_set():
-            log.info('Pointing series aborted.')
-        else:
-            log.info('Pointing series finished.')
+        log.info('Pointing series finished.')
 
     def _process_acquisition(self, datetime: str, ra: float, dec: float, alt: float, az: float,
                              off_ra: float = None, off_dec: float = None, off_alt: float = None, off_az: float = None):
