@@ -1,4 +1,4 @@
-from typing import List, Union
+from typing import List, Union, Any, Optional, Dict
 import logging
 
 from .publisher import Publisher
@@ -10,18 +10,18 @@ log = logging.getLogger(__name__)
 class MultiPublisher(Publisher):
     """Forwards a message to multiple publishers."""
 
-    def __init__(self, publishers: List[Union[Publisher, dict]] = None, *args, **kwargs):
+    def __init__(self, publishers: Optional[List[Union[Publisher, Dict[str, Any]]]] = None, **kwargs: Any):
         """Initialize new multi publisher.
 
         Args:
             publishers: Publishers to forwards messages to.
         """
-        Publisher.__init__(self, *args, **kwargs)
+        Publisher.__init__(self, **kwargs)
 
         # store
-        self._publishers: List[Publisher] = [] if publishers is None else [self._add_child_object(p) for p in publishers]
+        self._publishers: List[Publisher] = [] if publishers is None else [self.add_child_object(p) for p in publishers]
 
-    def __call__(self, **kwargs):
+    async def __call__(self, **kwargs: Any) -> None:
         """Publish the given results.
 
         Args:
@@ -31,7 +31,7 @@ class MultiPublisher(Publisher):
         # loop all publishers
         for p in self._publishers:
             # forward message
-            p(**kwargs)
+            await p(**kwargs)
 
 
 __all__ = ['MultiPublisher']
