@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import asyncio
 import threading
 import numpy as np
 from astropy.coordinates import SkyCoord
@@ -9,6 +10,8 @@ import random
 
 from pyobs.object import Object
 from pyobs.utils.enums import MotionStatus
+from pyobs.utils.parallel import event_wait
+
 if TYPE_CHECKING:
     from pyobs.utils.simulation import SimWorld
 
@@ -20,7 +23,7 @@ class SimTelescope(Object):
     def __init__(self, world: 'SimWorld', position: Optional[Tuple[float, float]] = None,
                  offsets: Optional[Tuple[float, float]] = None, pointing_offset: Optional[Tuple[float, float]] = None,
                  move_accuracy: float = 2., speed: float = 20., focus: float = 50, filters: Optional[List[str]] = None,
-                 filter: str = 'clear', drift: Optional[Tuple[float, float]] = None, focal_length: float = 5000.,
+                 filter_name: str = 'clear', drift: Optional[Tuple[float, float]] = None, focal_length: float = 5000.,
                  **kwargs: Any):
         """Initializes new telescope.
 
@@ -33,7 +36,7 @@ class SimTelescope(Object):
             speed: Speed of telescope in deg/sec.
             focus: Telescope focus.
             filters: List of filters.
-            filter: Current filter.
+            filter_name: Current filter.
             drift: RA/Dec drift of telescope in arcsec/sec.
             focal_length: Focal length of telescope in mm.
         """
@@ -53,7 +56,7 @@ class SimTelescope(Object):
         self.speed = speed     # telescope speed in deg/sec
         self.focus = focus
         self.filters = ['clear', 'B', 'V', 'R'] if filters is None else filters
-        self.filter = filter
+        self.filter_name = filter_name
         self.drift = (0., 0.) if drift is None else drift     # arcsec/sec in RA/Dec
         self.focal_length = focal_length
 
@@ -65,7 +68,7 @@ class SimTelescope(Object):
         self._pos_lock = threading.RLock()
 
         # threads
-        self.add_thread_func(self._move_thread)
+        self.add_background_task(self._move_task)
 
     @property
     def position(self) -> SkyCoord:
@@ -134,11 +137,11 @@ class SimTelescope(Object):
         # set offsets
         self._offsets = (ra, dec)
 
-    def _move_thread(self) -> None:
+    async def _move_task(self) -> None:
         """Move the telescope over time."""
 
         # run until closed
-        while not self.closing.is_set():
+        while True:
 
             # do we have destination coordinates?
             if self._dest_coords is not None:
@@ -186,7 +189,7 @@ class SimTelescope(Object):
                     self._drift = (self._drift[0] + drift_ra, self._drift[1] + drift_dec)
 
             # sleep a second
-            self.closing.wait(1)
+            await asyncio.sleep(1)
 
 
 __all__ = ['SimTelescope']
